﻿using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Common;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    public abstract class Expression : DeepCloneable
    {
        private string stringRepresentation;
        private int id;

        public Expression()
        { }

        /// <summary>
        /// Clone Constructor
        /// </summary>
        protected Expression(Expression origin, Cloner cloner)
        {
            stringRepresentation = origin.stringRepresentation;
            id = origin.id;
        }

        public abstract void Eval(IInterpreter interpreter);

        protected abstract int InitId();

        protected abstract string InitStringRepresentation();

        public string StringRepresentation
        {
            get
            {
                if (stringRepresentation == null)
                {
                    stringRepresentation = InitStringRepresentation();
                }

                return stringRepresentation;
            }
        }

        public int Id
        {
            get
            {
                if (id == default(int))
                {
                    id = InitId();
                }

                return id;
            }
        }

        public override bool Equals(object obj)
        {
            return obj != null && (GetHashCode() == obj.GetHashCode() || GetType() == obj.GetType());
        }

        public override string ToString()
        {
            return StringRepresentation;
        }

        public override int GetHashCode()
        {
            return Id;
        }
    }
}