﻿using HeuristicLab.Algorithms.PushGP.Generators;
using HeuristicLab.Algorithms.PushGP.Interpreter;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    /// <summary>
    /// Pushes a random NAME.
    /// </summary>
    public class NameRandExpression : StatelessExpression
    {
        protected override string InitStringRepresentation() { return "NAME.RAND"; }

        public override void Eval(IInterpreter interpreter)
        {
            var name = NameGenerator.RandomName();

            interpreter.NameStack.Push(name);
        }
    }

    /// <summary>
    /// Pushes a random integer.
    /// </summary>
    public class IntegerRandExpression : StatelessExpression
    {
        protected override string InitStringRepresentation() { return "INTEGER.RAND"; }

        public override void Eval(IInterpreter interpreter)
        {
            var value = IntegerGenerator.RandomInteger(
                interpreter.Configuration.MinRandomInteger,
                interpreter.Configuration.MaxRandomInteger);

            interpreter.IntegerStack.Push(value);
        }
    }

    /// <summary>
    /// Pushes a random float.
    /// </summary>
    public class FloatRandExpression : StatelessExpression
    {
        protected override string InitStringRepresentation() { return "FLOAT.RAND"; }

        public override void Eval(IInterpreter interpreter)
        {
            var value = FloatGenerator.RandomFloat(
                interpreter.Configuration.MinRandomFloat,
                interpreter.Configuration.MaxRandomFloat);

            interpreter.FloatStack.Push(value);
        }
    }

    /// <summary>
    /// Pushes a random boolean.
    /// </summary>
    public class BooleanRandExpression : StatelessExpression
    {
        protected override string InitStringRepresentation() { return "BOOLEAN.RAND"; }

        public override void Eval(IInterpreter interpreter)
        {
            var value = BooleanGenerator.RandomBoolean();

            interpreter.BooleanStack.Push(value);
        }
    }

    /// <summary>
    /// Pushes random expressions onto the code stack.
    /// </summary>
    public class CodeRandExpression : StatelessExpression
    {
        protected override string InitStringRepresentation() { return "CODE.RAND"; }

        public override void Eval(IInterpreter interpreter)
        {
            if (interpreter.IntegerStack.Count == 0 ||
                interpreter.IntegerStack.Top < 1)
                return;

            var size = (int)(interpreter.IntegerStack.Pop() % interpreter.Configuration.MaxPointsInRandomExpression);
            var expression = interpreter.CodeGenerator.RandomCode(size);
        }
    }
}
